var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _BaseController_dataModule, _BaseController_stateModule;
import DataController from "./DataController";
import StateController from "./StateController";
/**
 * Module with a dynamically updated data.
 * Manages the state and data variables of modules and stores.
 *
 * S — stored state type (this.state).
 *
 * D — stored data type (this.data).
 */
class BaseController {
    /**
     * Module with a dynamically updated data.
     * Manages the state and data variables of modules and stores.
     *
     * @param {TObject} defaultState - default value for the state.
     * @param {TObject} defaultData - default value for stored data.
     */
    constructor(defaultState, defaultData) {
        _BaseController_dataModule.set(this, void 0);
        _BaseController_stateModule.set(this, void 0);
        __classPrivateFieldSet(this, _BaseController_stateModule, new StateController(defaultState), "f");
        __classPrivateFieldSet(this, _BaseController_dataModule, new DataController(defaultData), "f");
    }
    /**
     * Method for getting current state.
     */
    get state() {
        return __classPrivateFieldGet(this, _BaseController_stateModule, "f").state;
    }
    /**
     * Method for getting the current data stored in the module.
     */
    get data() {
        return __classPrivateFieldGet(this, _BaseController_dataModule, "f").data;
    }
    /**
     * Method for changing one or more module states.
     *
     * @param {Partial<TObject> | keyof TObject} state - an object with new state data or a key.
     * @param {TObject[keyof TObject]} value - new value for state (if key is given).
     */
    setState(state, value) {
        __classPrivateFieldGet(this, _BaseController_stateModule, "f").setState(state, value);
    }
    /**
     * Method for resetting module state to its defaults.
     * @param {keyof TObject} keep - state keys that should not be reset.
     */
    resetState(...keep) {
        __classPrivateFieldGet(this, _BaseController_stateModule, "f").resetState(...keep);
    }
    /**
     * Method for changing one or more module states.
     *
     * @param {Partial<TObject> | keyof TObject} data - object with new data or key.
     * @param {TObject[keyof TObject]} value - new data value by key (if given).
     */
    setData(data, value) {
        __classPrivateFieldGet(this, _BaseController_dataModule, "f").setData(data, value);
    }
    /**
     * Method for resetting data stored by the module to its defaults.
     * @param {keyof TObject} keep - data keys that should not be reset.
     */
    resetData(...keep) {
        __classPrivateFieldGet(this, _BaseController_dataModule, "f").resetData(...keep);
    }
}
_BaseController_dataModule = new WeakMap(), _BaseController_stateModule = new WeakMap();
export default BaseController;
